package bitel.billing.test.convert;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import bitel.billing.common.CommonUtils;
import bitel.billing.common.SimpleSetup;
import bitel.billing.common.TimeUtils;
import bitel.billing.server.contract.ContractRemover;
import bitel.billing.server.contract.bean.ContractMemo;
import bitel.billing.server.contract.bean.ContractMemoManager;
import bitel.billing.server.contract.bean.ContractParamUtils;
import bitel.billing.server.contract.object.bean.DateParamValue;
import bitel.billing.server.contract.object.bean.ListParamValue;
import bitel.billing.server.contract.object.bean.ParamValueManager;
import bitel.billing.server.contract.object.bean.TextParamValue;
import bitel.billing.server.script.bean.event.Event;
import bitel.billing.server.script.bean.event.EventProcessor;
import bitel.billing.server.script.bean.event.ExtraEventProcessor;
import bitel.billing.server.tariff.TariffTreeBuilder;
import bitel.billing.server.util.DefaultServerSetup;
import bitel.billing.server.util.Utils;

public class ConvertUtil
{
    /**
     * 
     * @param group    !
     * @param setup 
     */
	public static void removeContracts( long group, DefaultServerSetup setup )
	{
		Connection con = null;		
		try
		{
			System.out.println( " removing contracts ");
			con = setup.getDBConnection();
			
			ContractRemover remover = new ContractRemover( setup, con, null );
			
			String query = "SELECT	id FROM  contract WHERE gr & ? != 0 ";
			PreparedStatement ps  = con.prepareStatement( query );
			ps.setLong( 1, group );
			ResultSet rs = ps.executeQuery();
						
			
			int count = 0;
			while ( rs.next() )
			{
				int cid = rs.getInt( 1 );
				remover.deleteContract( cid );
				count ++;
				
				
			}
			rs.close();
			ps.close();
			
			System.out.println( count + " contracts was deleted..");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			//setup.returnConToPool( con );
			closeConnection( con );
		}
		
		
	}

	public static void closeConnection( Connection con )
	{
		try
		{
			con.close();
		} catch (SQLException e)
		{			
		}
	}		
	
	public  static Map<String, Integer >  getBillingTariffMap( DefaultServerSetup setup )
	{
		Map<String, Integer > tarifMap = new HashMap<String, Integer>();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT id, title FROM tariff_plan";
			
			 
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				tarifMap.put( rs.getString( 2 ).toLowerCase(), rs.getInt( 1 )  );
				
			}
			rs.close();
			ps.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		
		return tarifMap; 
	}	
	
	public  static Map<String, Integer >  getBillingCityMap( DefaultServerSetup setup )
	{
		Map<String, Integer > tarifMap = new HashMap<String, Integer>();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT id, title FROM address_city";
			
			 
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				tarifMap.put( rs.getString( 2 ), rs.getInt( 1 )  );
				
			}
			rs.close();
			ps.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		return tarifMap; 
	}			
	
	public  static Map<String, Integer >  getBillingStreetMap( DefaultServerSetup setup )
	{
		Map<String, Integer > tarifMap = new HashMap<String, Integer>();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT id, title FROM address_street";
			
			 
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				tarifMap.put( rs.getString( 2 ), rs.getInt( 1 )  );
				
			}
			rs.close();
			ps.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		return tarifMap; 
	}		
	
	/**
     *   - .
     * @param cid
     * @param con
     * @param value ,  null/ -  ,   . 
     * @param pid ,   - .
     * @throws SQLException
     */
	public static void addContactParamString(int cid, Connection con,  String value, int pid )
	{
		
		try
		{
			
			String query = "DELETE FROM contract_parameter_type_1 WHERE cid=? AND pid=?";
			PreparedStatement ps =  con.prepareStatement( query );
			ps.setInt( 1, cid );
			ps.setInt( 2, pid );
			ps.executeUpdate();
			ps.close();

			if ( !Utils.isEmptyString( value ) )
			{
				query = "INSERT INTO contract_parameter_type_1 " +
						"( cid, pid, val ) VALUES ( ?, ?, ? )";
				ps = con.prepareStatement( query );
				ps.setInt( 1, cid );
				ps.setInt( 2, pid );
				ps.setString( 3, value );
				ps.executeUpdate();
				ps.close();
			}			
			
			
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{			
		}
	}	
	
	/**
     *   - .
     * @param cid
     * @param con
     * @param date ,  null -  ,   . 
     * @param datePid  ,   - .
     * @throws SQLException
     */
    public static void addContactParamDate( int cid, Connection con, Date date, int datePid ) throws SQLException
    {
        if( datePid > 0 )
        {
            String query = "DELETE FROM contract_parameter_type_6 WHERE cid=? AND pid=?";
            PreparedStatement ps =  con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, datePid );
            ps.executeUpdate();
            ps.close();
            if ( date != null )
            {
                query = "INSERT INTO contract_parameter_type_6 ( cid, pid, val ) VALUES ( ?, ?, ? )";
                ps = con.prepareStatement( query );
                ps.setInt( 1, cid );
                ps.setInt( 2, datePid );
                ps.setDate( 3, TimeUtils.convertDateToSqlDate( date ) );
                ps.executeUpdate();
                ps.close();
            }
        }
    }
    
    /**
     *   - .
     * @param cid
     * @param con
     * @param email ,  null -  ,   . 
     * @param emailPid ,   - .
     * @throws SQLException
     */
    public static void addContactParamEmail( int cid,
                                             Connection con,
                                             String email,
                                             int emailPid ) throws SQLException
    {
        if( emailPid > 0 )
        {            
            String query = "DELETE FROM contract_parameter_type_3 WHERE cid=? AND pid=?";
            PreparedStatement ps =  con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, emailPid );
            ps.executeUpdate();
            ps.close();
            if ( email != null )
            {
                query = "INSERT INTO contract_parameter_type_3 ( cid, pid, email ) VALUES ( ?, ?, ? )";
                ps = con.prepareStatement( query );
                ps.setInt( 1, cid );
                ps.setInt( 2, emailPid );
                ps.setString( 3, email );
                ps.executeUpdate();
                ps.close();
            }
        }
    }
    
    public static void addContactParamFlag( int cid, Connection con, boolean flag, int flagPid ) throws SQLException
    {
        if ( flagPid > 0 )
        {
            String query = "DELETE FROM contract_parameter_type_5 WHERE cid=? AND pid=?";
            PreparedStatement ps = con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, flagPid );
            ps.executeUpdate();
            ps.close();
            //
            query = "INSERT INTO contract_parameter_type_5 ( cid, pid, val ) VALUES ( ?, ?, ? )";
            ps = con.prepareStatement( query );
            ps.setInt( 1, cid );
            ps.setInt( 2, flagPid );
            ps.setBoolean( 3, flag );
            ps.executeUpdate();
            ps.close();
        }
    }
	
	public static void addObjectParamString( int objectId, Connection con,  String value, int pid )
	{
		
		try
		{
			
			TextParamValue paramValue = new TextParamValue();
			paramValue.setParamId( pid);
			paramValue.setValue( value );
			paramValue.setObjectId( objectId );
			
			new ParamValueManager( con ).updateParamValue( paramValue );
			
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{			
		}		
				
		
	}			
	
	public static void addObjectParamDate( int objectId, Connection con,  Calendar date, int pid )
	{
		
		try
		{			
			DateParamValue paramValue = new DateParamValue();			
			paramValue.setParamId( pid );
			paramValue.setValue( date );
			paramValue.setObjectId( objectId );
			
			new ParamValueManager( con ).updateParamValue( paramValue );
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{			
		}		
	}
	
	public static void addObjectParamList( int objectId, Connection con,  int  value, int pid )
	{
		
		try
		{			
			ListParamValue paramValue = new ListParamValue();
			paramValue.setObjectId( objectId );
			paramValue.setParamId( pid );
			paramValue.setValue( value );
			
			new ParamValueManager( con ).updateParamValue( paramValue );
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{			
		}		
	}
	
	public static void addContrtactParamList( int cid, Connection con,  int  value, int pid )
	{
		
		try
		{			
			new ContractParamUtils( con ).setListParam( cid, pid, value );			
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{			
		}		
	}	
	public static int addNewTariff( Connection con, String title ) throws SQLException
	{
		String query;
		PreparedStatement ps;
		//   
		query = "INSERT INTO tariff_plan ( title ) values ( ? )";
		ps = con.prepareStatement( query );
		ps.setString( 1,  title); 
		ps.executeUpdate();

		int tpId = lastInsertId( con );
		int treeId = new TariffTreeBuilder( con ).createTree();
		
		ps.close();
		
		query = "INSERT INTO tariff_tree_link (tpid, tree_id) VALUES (?, ?)";
		ps = con.prepareStatement( query );
		
		ps.setInt( 1, tpId );
		ps.setInt( 2, treeId );
		
		ps.executeUpdate();
		ps.close();
		return tpId;
	}	

	 public static int lastInsertId( Connection con ) throws SQLException
	 {
	 	int id = -1;
	 	
	 	PreparedStatement ps =  con.prepareStatement(   "SELECT last_insert_id() as id" );
	 	
	    ResultSet rs = ps.executeQuery( "SELECT last_insert_id() as id" );
	    while( rs.next() )
	    {
	    	id = rs.getInt( 1 );
	    }
	    rs.close();
	    ps.close();
	    return id;
	}		
	
	/**
	 *     
	 * @param filePath    
	 * @param charsetName  
	 * @param splitRegexp     - 
	 * @param fields     (   )
	 * @return  ( =>)
	 * @throws UnsupportedEncodingException
	 * @throws IOException
	 */
	public static List<Map<String, String>> loadTxtDB( String filePath,
                                                       String charsetName,
                                                       String splitRegexp,
                                                       String... fields )
        throws UnsupportedEncodingException, IOException
    {
        List<Map<String, String>> result = new ArrayList<Map<String, String>>();
        InputStream stream = new FileInputStream( filePath );
        //
        int count = 0;
        byte buff[] = new byte[1024];
        StringBuffer strBuffer = new StringBuffer();
        //
        while ( (count = stream.read( buff, 0, 1024 )) > 0 )
        {
            strBuffer.append( new String( buff, 0, count, charsetName ) );
        }
        String lines[] = strBuffer.toString().split( "\n" );
        for ( String line : lines )
        {
            Map<String, String> map = new HashMap<String, String>();
            String words[] = line.split( splitRegexp );
            int idx = 0;
            for ( String fieldName : fields )
            {
                map.put( fieldName, words[idx++].trim() );
            }
            result.add( map );
        }
        return result;
    }
	
	/**
	 *    .   TelnetTV,    . 
	 * @param setup
	 * @return    (_=>(_[/_]=>_)
	 */
	public static Map<Integer, Map<String, Integer>> getBillingHouseMap( DefaultServerSetup setup )
    {
        Map<Integer, Map<String, Integer>> result = new HashMap<Integer, Map<String, Integer>>();
        Connection con = null;
        try
        {
            con = setup.getDBConnectionFromPool();
            String query = "SELECT streetid, house, id, frac FROM address_house";
            PreparedStatement ps = con.prepareStatement( query );
            ResultSet rs = ps.executeQuery();
            while ( rs.next() )
            {
                int steetId = rs.getInt( 1 );
                int house = rs.getInt( 2 );
                int houseId = rs.getInt( 3 );
                String frac = rs.getString( 4 );
                Map<String, Integer> houses = result.get( steetId );
                if ( houses == null )
                {
                    houses = new HashMap<String, Integer>();
                    result.put( steetId, houses );
                }
                houses.put( String.valueOf( house ) + ((frac!=null && !frac.isEmpty())?("/" + String.valueOf( frac )):("")) , houseId );
            }
            rs.close();
            ps.close();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        finally
        {
            setup.returnConToPool( con );
        }
        return result;
    } 
    
    //  map1    map2    
	public static <K extends Object, V extends Object> void combineMaps ( Map<String, Map<K,V> > map1, Map<String, Map<K, V> > map2 )
	{
	    for (String key  : map1.keySet())
	    {
	    	
	    	Map<K, V> values1  =  map1.get( key );
	    	
	    	Map<K, V> values2  =  map2.get( key );
	    	
	    	values1.putAll( values2 );
	    	
	    }
    }	
	
	public static String getHouseKey( int house, String frac )
	{
		String result  = String.valueOf( house );
		
		if ( !CommonUtils.isEmptyString( frac ))
		{
			result += "##" + frac;
		}
		
		return result;
	}		
			
	public  static Map<Integer, Map<String, Integer> >  geHouseMap( DefaultServerSetup setup )
	{
		Map<Integer, Map<String, Integer> > result = new HashMap<Integer, Map<String, Integer> >();
		Connection con = null;
		try
		{
			con = setup.getDBConnectionFromPool();
			
			String query = "SELECT streetid, house, id, frac FROM address_house";
			
			
			PreparedStatement ps =  con.prepareStatement(   query );            
			ResultSet rs = ps.executeQuery();
			while ( rs.next() )
			{
				int steetId = rs.getInt( 1 );
				int house = rs.getInt( 2 );
				int houseId = rs.getInt( 3 );
				String frac = rs.getString( 4 );
				
				
				Map<String,Integer> houses = result.get( steetId );
				if ( houses == null)
				{
					houses = new HashMap<String, Integer>();
					result.put( steetId, houses );
				}
				houses.put ( getHouseKey( house, frac ) , houseId );
				
			}
			rs.close();
		} catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			setup.returnConToPool( con );
		}
		
		
		return result; 
	}		

    /**
     *    .
     * @param cid
     * @param groupBitmask
     * @param con
     * @throws SQLException
     */
    public static void addContactGroup( int cid, Long groupBitmask, Connection con ) throws SQLException
    {
        if( groupBitmask != null )
        {
            String query = "UPDATE contract SET gr=gr|? WHERE id=?";
            PreparedStatement ps = con.prepareStatement( query );
            ps.setLong( 1, groupBitmask );
            ps.setInt( 2, cid );
            ps.executeUpdate();
            ps.close();
        }
    }
    
    public static void updateContactComment( int cid, String comment, Connection con ) throws SQLException, UnsupportedEncodingException
    {
        String query = "UPDATE contract SET comment=? WHERE id=?";
        PreparedStatement ps = con.prepareStatement( query );
        ps.setString( 1, comment );
        ps.setInt( 2, cid );
        ps.executeUpdate();
        ps.close();
    }
    
    public static void updateContactClosesumma( int cid, double closesumma, Connection con )
        throws SQLException, UnsupportedEncodingException
    {
        String query = "UPDATE contract SET closesumma=? WHERE id=?";
        PreparedStatement ps = con.prepareStatement( query );
        ps.setDouble( 1, closesumma );
        ps.setInt( 2, cid );
        ps.executeUpdate();
        ps.close();
    }
    
    /**
     *    - 
     * @param number
     * @param nullCount
     * @return
     */
    static String addNull( int number, int nullCount )
    {
        StringBuilder sb = new StringBuilder(nullCount);
        String nums = String.valueOf( number );
        for( int i=0; i<nullCount-nums.length(); i++ )
        {
            sb.append( "0" );
        }
        sb.append( nums );
        return sb.toString();
    }
    
    static void updateComments( Connection bgCon, int cid, String comments ) throws UnsupportedEncodingException, SQLException
    {        
        if( CommonUtils.notBlankString( comments ) )
        {
            ContractMemoManager memoManager = new ContractMemoManager( bgCon );
            ContractMemo memo = new ContractMemo();
            memo.setUserId( 0 );
            memo.setContractId( cid );
            memo.setComment( comments );
            memo.setSubject( "" );
            memoManager.updateMemo( memo );
        }
    }

    public static void addTariff( int cid, int tpid, Calendar date1, Connection con ) throws SQLException
    {
        String query = "INSERT INTO contract_tariff( cid, tpid, date1 ) VALUES ( ?, ?, ?)";
        PreparedStatement ps = con.prepareStatement( query );
        ps.setInt( 1, cid );
        ps.setInt( 2, tpid );
        ps.setDate( 3, TimeUtils.convertCalendarToSqlDate( date1 ) );
        ps.executeUpdate();
        ps.close();
    }
   
    /**
     *   Calendar  java.sql.Date
     * 
     * @param calendar
     *             
     * @return java.sql.Date,  calendar != null,  null
     */
    public static final java.sql.Date convertCalendarToSqlDate( Calendar calendar )
    {
        java.sql.Date result = null;
        if ( calendar != null )
        {
            result = new java.sql.Date( calendar.getTimeInMillis() );
        }
        return result;
    }

    /**
     *    ( )
     * @param cid 
     * @param con 
     * @param password 
     * @throws SQLException 
     */
	public static void setWebStatPassword( int cid, Connection con, String password ) throws SQLException
	{
		String query = "UPDATE contract SET pswd=? WHERE id=?";
		PreparedStatement ps = con.prepareStatement( query );
		ps.setString( 1, password.trim() );
		ps.setInt( 2, cid );
		ps.executeUpdate();
		ps.close();
		//
		query = "INSERT INTO log_contract_pswd VALUES ( now(), ?, ? )";
		ps = con.prepareStatement( query );
		ps.setInt( 1, 0 );
		ps.setInt( 2, cid );
		ps.executeUpdate();
		ps.close();
	}
	 
}

//TODO -    4.6 .  Setup,     .        ,     
//   ,  class SetupData   DefaultServerSetup -       .,         
class FileSetup extends  SimpleSetup
{
	    public FileSetup( String bundleName )
	    {
	        init( bundleName);
	    }

	    private void init( String bundleName )
	    {
	        //   
	        System.setProperty( "javax.xml.transform.TransformerFactory", "org.apache.xalan.processor.TransformerFactoryImpl" );

	        ResourceBundle rb = ResourceBundle.getBundle( bundleName );
	        for( String key : rb.keySet() )
	        {
	            setValue( key, rb.getObject( key ) );
	        }
	    }
	    
}

//,     
//   ..      -   
//-     
class ConvertEventProcessor extends EventProcessor
{

	@Override
	public void addEvent( Event event )
	{
		//   
	}
	
	public static void initProcessor( DefaultServerSetup setup, boolean processMode, ExtraEventProcessor extraProccessor )
	{
		processor = new ConvertEventProcessor( setup, processMode, extraProccessor);
	}
	

	protected ConvertEventProcessor( DefaultServerSetup setup, boolean processMode, ExtraEventProcessor extraProccessor )
	{
		super( setup, processMode, extraProccessor );		
	}
	
}