package bitel.billing.test.convert;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import bitel.billing.server.contract.bean.ContractAddressParamValue;
import bitel.billing.server.contract.bean.ContractParamUtils;
import bitel.billing.server.util.SetupData;
import bitel.billing.server.util.Utils;

import com.Ostermiller.util.ExcelCSVParser;

public class TkUralAddressSet
{
    private static final String ABONENT_ADDRESS_FILE = "/home/shamil/work/tkural/1/abonent_address.csv";
    private static final String STREET_INDEX_FILE = "/home/shamil/work/tkural/1/index.csv";
    private static final String LOG_FILE = "/home/shamil/work/tkural/address.log";
    
    private static final int ADDRESS_PARAM = 21;
    
    private SetupData setup = new SetupData( "bitel.billing.test.convert.tkural" );
    private Connection con;
        
    public TkUralAddressSet()
    {
        try 
        {
            loadStreetIndex();
            
            BufferedWriter log = new BufferedWriter( new OutputStreamWriter( new FileOutputStream( LOG_FILE ), "Cp1251" ) );
            
            con = setup.getDBConnectionFromPool();
            
            String query = 
                "SELECT contract.id " +
                "FROM phone_item_number_1 AS item_number " +
                "INNER JOIN phone_client_item_1 AS client_item ON item_number.item_id=client_item.id " +
                "INNER JOIN contract ON client_item.cid=contract.id " +
                "WHERE item_number.number=? AND contract.comment=?";
            PreparedStatement psFindContract = con.prepareStatement( query );            
            
            ContractParamUtils cpu = new ContractParamUtils( con );
            
            int rowNum = 0;
            int notFoundCount = 0;
                        
            String[][] data = ExcelCSVParser.parse( new InputStreamReader( new FileInputStream( ABONENT_ADDRESS_FILE ), "Cp1251" ) , ';' );
            if( data != null && data.length > 0 )
            {
                final int rowCount = data.length;
                for( rowNum = 1; rowNum < rowCount; rowNum++ )
                {
                    String[] fields = data[rowNum];
                    
                    if( fields.length < 8 )
                    {
                        System.out.println( "Skipping row: " + rowNum );
                        continue;
                    }

                    String contract = fields[0].trim();
                    String number = fields[1];
                    String fio = fields[2];
                    String street = fields[3];
                    int house = Utils.parseIntString( fields[4].trim() );
                    String frac = fields[5];
                    String flat = fields[6];
                    String room = fields[7];
                   
                    if( Utils.parseLongString( number.trim() ) == 0 )
                    {
                        continue;
                    }
                    
                    fio = fio.trim();
                    fio = fio.replaceAll( "  ", " " ).replaceAll( "  ", " " ).replaceAll( "  ", " " );
                    
                    frac = frac.replaceAll( "-", "" ).trim();
                    room = room.replaceAll( "-", "" ).trim();
                    
                    psFindContract.setString( 1, "7343" + number );
                    psFindContract.setString( 2, fio );
                    
                    String message = null;
                    if( house <= 0 )
                    {
                        message =  
                            "Incorrect house number row " + rowNum + " n " + number + " fio " + fio  + " s " + street + " h " + house + " fr " + frac + 
                            " flat " + flat + " r " + room; 
                        
                        System.out.println( message );
                        log.write( message );
                        log.write( "\n" );
                        
                        continue;
                    }
                    
                    String index = findIndex( street, String.valueOf( house ), frac );
                    if( index == null )
                    {
                        message = "Index not found: street " + street + " house " + house + " frac " + frac; 
                        
                        System.out.println( message );
                        log.write( message );
                        log.write( "\n" );
                    }
                                        
                    ResultSet rs = psFindContract.executeQuery();
                    if( rs.next() )
                    {
                        int cid = rs.getInt( 1 );
                        
                        String addressString = index + " " + street + " " + house;
                        if( Utils.notBlankString( frac ) )
                        {
                            addressString += "/" + frac;
                        }
                        addressString += " ." + flat;
                        if( Utils.notBlankString( room ) )
                        {
                            addressString += " . " + room;
                        }
                        
                        int streetId = getStreetId( street );
                        int houseId = getHouseId( streetId, house, frac, index );
                        
                        ContractAddressParamValue apv = new ContractAddressParamValue();
                        apv.setHouseId( houseId );
                        apv.setFlat( flat );
                        apv.setRoom( room );                        
                        apv.setAddress( addressString );
                        
                        cpu.setAddressParam( cid, ADDRESS_PARAM, apv );                        
                    }
                    else
                    {
                        message =  
                            "Contract not found: row " + rowNum + " n " + number + " fio " + fio  + " s " + street + " h " + house + " fr " + frac + 
                            " flat " + flat + " r " + room;

                        System.out.println( message );
                        log.write( message );
                        log.write( "\n" );

                        notFoundCount++;
                    }
                    rs.close();
                    
                    if( rowNum > 0 &&
                        rowNum % 100 == 0 )
                    {
                        System.out.println( "row " + rowNum );
                    }
                    
                    /*if( rowNum > 100 )
                    {
                        System.out.println( "Break on 100" );
                        break;
                    }*/
                }
            }
            
            log.flush();
            
            System.out.println( "Not found: " + notFoundCount );
        }        
        catch (Exception e) 
        {
            e.printStackTrace();
        }
    }    
    
    private void loadStreetIndex()
        throws Exception
    {
        String[][] data = ExcelCSVParser.parse( new InputStreamReader( new FileInputStream( STREET_INDEX_FILE ), "Cp1251" ) , ',' );
        if( data != null && data.length > 0 )
        {
            final int rowCount = data.length;
            for( int rowNum = 1; rowNum < rowCount; rowNum++ )
            {
                String[] fields = data[rowNum];
                
                if( fields.length != 7 )
                {
                    System.out.println( "Skipping street index row: " + rowNum );
                    continue;
                }
                
                String street = fields[0];
                String oddFrom = fields[1];
                String oddTo = fields[2];
                String evenFrom = fields[3];
                String evenTo = fields[4];                
                String index = fields[6];
                
               /* if( street.equals( "" ) )
                {
                    System.out.println( "a" );
                }*/
                
                StreetIndex streetIndex = new StreetIndex();
                streetIndex.evenFrom = parseVal( evenFrom );
                streetIndex.evenTo = parseVal( evenTo );
                if( streetIndex.evenFrom == 0 )
                {
                    streetIndex.homesEven = Utils.stringToList( evenFrom );
                }
                
                streetIndex.oddFrom = parseVal( oddFrom );
                streetIndex.oddTo = parseVal( oddTo );
                if( streetIndex.oddFrom == 0 )
                {
                    streetIndex.homesOdd = Utils.stringToList( oddFrom );
                }
                streetIndex.index = index;                
                
                List<StreetIndex> indexList = streetMap.get( street );
                if( indexList == null )
                {
                    indexList = new ArrayList<StreetIndex>();
                    streetMap.put( street, indexList );
                }
                indexList.add( streetIndex );
            }
        }
    }
    
    private int parseVal( String val )
    {
        int result = 0;
        
        if( val.equals( "" ) )
        {
            result = StreetIndex.NO;
        }
        else if ( val.equals( "" ) ||  val.equals( " " ) )
        {
            result = StreetIndex.ALL;
        }
        else
        {
            result = Utils.parseIntString( val );
        }        
        
        return result;
    }
    
    private String findIndex( String street, String house, String frac )
        throws Exception
    {
        StreetIndex result = null;

        List<StreetIndex> indexList = streetMap.get( street );
        if( indexList == null )
        {
            return null;
            //throw new Exception( "Index not found street " + street );
        }
        
        int houseNumber = Utils.parseIntString( house );
        String houseWithFrac = String.valueOf( house );
        
        if( Utils.notBlankString( frac ) && !frac.equals( "-" ) )
        {
            houseWithFrac += frac;
        }
        
        /*if( street.equals( "" ) && houseNumber == 16 )
        {
            System.out.println( "a" );
        }*/
        
        for( StreetIndex index : indexList )
        {
            if( index.homesEven != null && index.homesEven.contains( houseWithFrac ) )
            {
                result = index;
                break;
            }
            if( index.homesOdd != null && index.homesOdd.contains( houseWithFrac ) )
            {
                result = index;
                break;
            }
            
            int rangeFrom = 0, rangeTo = 0;
            
            // 
            if( houseNumber % 2 == 0 )
            {
                rangeFrom = index.evenFrom;
                rangeTo = index.evenTo;
            }
            // 
            else
            {
                rangeFrom = index.oddFrom;
                rangeTo = index.oddTo;
            }
            
            if( rangeFrom == StreetIndex.NO )
            {
                continue;
            }
            if( rangeFrom == StreetIndex.ALL ||
                (rangeFrom <= houseNumber && (rangeTo == StreetIndex.ALL || houseNumber <= rangeTo) )
               )
            {
                result = index;
                break;
            }
        }
        
        return result != null ? result.index : null;
    }
    
    private Map<String, List<StreetIndex>> streetMap = new HashMap<String, List<StreetIndex>>();
    
    private static class StreetIndex
    {
        public static final int ALL = -1;
        public static final int NO = -2;
        
        public int oddFrom;
        public int oddTo;
        public int evenFrom;
        public int evenTo;
        public List<String> homesEven;
        public List<String> homesOdd;
        public String index;
    }
    
    private Map<String, Integer> streetCodes;
    
    private int getStreetId( String streetName )
        throws Exception
    {
        Integer result = null;
        String query = null;
        
        if( streetCodes == null )
        {
            streetCodes = new HashMap<String, Integer>();

            query = "SELECT title, id FROM address_street";
            PreparedStatement psSelectStreet = con.prepareStatement( query );
            ResultSet rs = psSelectStreet.executeQuery();
            while( rs.next() )
            {
                streetCodes.put( rs.getString( 1 ), rs.getInt( 2 ) );
            }
            psSelectStreet.close();
        }
        
        result = streetCodes.get( streetName );
        if( result == null )
        {        
            query = "INSERT INTO address_street (cityid, title) VALUES (1, ?)";
            PreparedStatement psInsertStreet = con.prepareStatement( query );
            psInsertStreet.setString( 1, streetName );
            psInsertStreet.executeUpdate();
            
            result = Utils.lastInsertId( psInsertStreet );
            psInsertStreet.close();
            
            streetCodes.put( streetName, result );
        }

        return result;
    }
    
    private int getHouseId( int streetId, int houseNumber, String frac, String index )
        throws Exception
    {
        Integer result = null;
        
        String query = "SELECT id FROM address_house WHERE cityid=1 AND streetid=? AND house=? AND frac=? LIMIT 1";
        PreparedStatement psFindHouse = con.prepareStatement( query );
        psFindHouse.setInt( 1, streetId );
        psFindHouse.setInt( 2, houseNumber );
        psFindHouse.setString( 3, frac );
        
        ResultSet rs = psFindHouse.executeQuery();
        if( rs.next() )
        {
            result = rs.getInt( 1 );
        }
        else
        {
            query = "INSERT INTO address_house (cityid, streetid, house, frac, box_index) VALUES (1, ?, ?, ?, ?)";
            PreparedStatement psInsertHouse = con.prepareStatement( query );
            psInsertHouse.setInt( 1, streetId );
            psInsertHouse.setInt( 2, houseNumber );
            psInsertHouse.setString( 3, frac );
            psInsertHouse.setString( 4, index );
            
            psInsertHouse.executeUpdate();
            result = Utils.lastInsertId( psInsertHouse );
            psInsertHouse.close();            
        }
        psFindHouse.close();
        
        return result;
    }
    
    public static void main( String[] args )
    {
        new TkUralAddressSet();
    }
}